@@ -2,7 +2,7 @@
 /*
   Copyright(C) 2010 Tetsuro IKEDA
   Copyright(C) 2010-2013 Kentoku SHIBA
-  Copyright(C) 2011-2015 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2011-2018 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,9 @@
 // for debug
 #define MRN_CLASS_NAME "mrn::DatabaseManager"
 
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #ifdef WIN32
 #  include <direct.h>
 #  define MRN_MKDIR(pathname, mode) _mkdir((pathname))
@@ -40,6 +43,8 @@
 #  define MRN_MKDIR(pathname, mode) mkdir((pathname), (mode))
 #endif
 
+#include <cstring>
+
 extern "C" {
   grn_rc GRN_PLUGIN_IMPL_NAME_TAGGED(init, normalizers_mysql)(grn_ctx *ctx);
   grn_rc GRN_PLUGIN_IMPL_NAME_TAGGED(register, normalizers_mysql)(grn_ctx *ctx);
@@ -57,7 +62,7 @@ namespace mrn {
       void *db_address;
       GRN_HASH_EACH(ctx_, cache_, id, NULL, 0, &db_address, {
         Database *db;
-        memcpy(&db, db_address, sizeof(grn_obj *));
+        grn_memcpy(&db, db_address, sizeof(db));
         delete db;
       });
       grn_hash_close(ctx_, cache_);
@@ -69,7 +74,7 @@ namespace mrn {
     cache_ = grn_hash_create(ctx_,
                              NULL,
                              GRN_TABLE_MAX_KEY_SIZE,
-                             sizeof(grn_obj *),
+                             sizeof(Database *),
                              GRN_OBJ_KEY_VAR_SIZE);
     if (!cache_) {
       GRN_LOG(ctx_, GRN_LOG_ERROR,
@@ -127,7 +132,7 @@ namespace mrn {
       grn_hash_add(ctx_, cache_,
                    mapper.db_name(), strlen(mapper.db_name()),
                    &db_address, NULL);
-      memcpy(db_address, db, sizeof(Database *));
+      grn_memcpy(db_address, db, sizeof(Database *));
       error = ensure_normalizers_registered((*db)->get());
       if (!error) {
         if ((*db)->is_broken()) {
@@ -146,7 +151,7 @@ namespace mrn {
         }
       }
     } else {
-      memcpy(db, db_address, sizeof(Database *));
+      grn_memcpy(db, db_address, sizeof(Database *));
       grn_ctx_use(ctx_, (*db)->get());
     }
 
@@ -169,7 +174,8 @@ namespace mrn {
     }
 
     Database *db = NULL;
-    memcpy(&db, db_address, sizeof(Database *));
+    grn_memcpy(&db, db_address, sizeof(db));
+    grn_ctx_use(ctx_, db->get());
     if (db) {
       delete db;
     }
@@ -199,7 +205,8 @@ namespace mrn {
         db = new Database(ctx_, grn_db);
       }
     } else {
-      memcpy(&db, db_address, sizeof(Database *));
+      grn_memcpy(&db, db_address, sizeof(db));
+      grn_ctx_use(ctx_, db->get());
     }
 
     if (!db) {
@@ -248,7 +255,8 @@ namespace mrn {
       void *db_address;
       Database *db;
       grn_hash_cursor_get_value(ctx_, cursor, &db_address);
-      memcpy(&db, db_address, sizeof(Database *));
+      grn_memcpy(&db, db_address, sizeof(db));
+      grn_ctx_use(ctx_, db->get());
       grn_rc rc = grn_hash_cursor_delete(ctx_, cursor, NULL);
       if (rc) {
         error = ER_ERROR_ON_READ;
@@ -262,6 +270,11 @@ namespace mrn {
     DBUG_RETURN(error);
   }
 
+  const char *DatabaseManager::error_message() {
+    MRN_DBUG_ENTER_METHOD();
+    DBUG_RETURN(ctx_->errbuf);
+  }
+
   void DatabaseManager::mkdir_p(const char *directory) {
     MRN_DBUG_ENTER_METHOD();
 
